/*
 * flash_watch.cc - OSD-Flash chip control sample program
 *
 * Copyright 2017 Sony Coporation
*/

#include <stdio.h>    // printf
#include <unistd.h>
#include <sys/ioctl.h>    // ioctl, _IOR, _IOWR

#include "flash_control.h"
#include "OSD_flash.h"

void usage( void )
{
	printf("usage:\n");
	printf("<cmd> [<addr> <data>]\n");
	printf("  j .................. Manufacturer ID & Device ID\n");
	printf("  s .................. Volatile Status Register Enable\n");
	printf("  e .................. Write Enable\n");
	printf("  d .................. Write Disable\n");
	printf("  p .................. Flash Dump\n");
	printf("  z .................. Chip Erase\n");
	printf("  1 .................. Read Status 1\n");
	printf("  2 .................. Read Status 2\n");
	printf("  3 .................. Read Status 3\n");
	printf("  7 .................. Write Status 1\n");
	printf("  8 .................. Write Status 2\n");
	printf("  9 .................. Write Status 3\n");
	printf("  q .................. quit\n");
	printf("\n");
}

// test DIR register reading/writing by command line request
int get_and_exec_cmd( FlashControl* dirc )
{
	uint32_t addr = 0;
	uint32_t lval = 0;
	uint8_t data = 0;
	char cmd_char = 0;
	char buff[256];
	char *retb;
	int num_params = 0;
	int ret;
	int cnt;
	bool flag;
	uint8_t  cmd;
	uint32_t status;

	printf( "input: <cmd> [<addr> [<write data>]]\n = " );
	retb = fgets( buff, sizeof(buff), stdin );
	if( retb ) {
		num_params = sscanf( buff, "%c%X%X", &cmd_char, &addr, &lval );
		data = (uint8_t)lval;
	}
	if( num_params <= 0 ) {
		return 1;
	}

	/* Open */
	ret = dirc->spiOpen();
	if( ret ) {
		printf( "SPI Open Error\n" );
		return 0;
	}

	switch( cmd_char ) {

		case 'h':
			usage();
			break;

		case 'j':
		{
			uint32_t m_id = 0;
			uint32_t d_id = 0;
			ret = dirc->getJedecId( &m_id, &d_id );
			printf( "get Manufacturer ID = 0x%02X : Device ID = 0x%04X\n", m_id, d_id );
			break;
		}

		case 'e':

			cmd = FLASH_SET_SR_WRITE_ENABLE;
			ret = dirc->setCommand( cmd );
			printf( "set Write Enable\n" );
			break;

		case 'd':

			cmd = FLASH_SET_WRITE_DISABLE;
			ret = dirc->setCommand( cmd );
			printf( "set Write Disable\n" );
			break;

		case 'p':

			ret = dirc->getData( addr );
			break;

		case 's':

			cmd = FLASH_SET_SR_WRITE_ENABLE;
			ret = dirc->setCommand( cmd );
			printf( "set Volatile Status Register Enable\n" );
			break;

		case 'z':

			printf( "Chip Erase Start\n" );
			cmd = FLASH_CHIP_ERASE;
			ret = dirc->setCommand( cmd );
			printf( "Chip Erase End\n" );
			break;

		case '1':

			cmd    = FLASH_READ_STATUS1;
			status = 0;
			ret = dirc->getStatus( cmd, &status );
			printf( "get STATUS 1 = 0x%02X\n", status );
			break;

		case '2':

			cmd    = FLASH_READ_STATUS2;
			status = 0;
			ret = dirc->getStatus( cmd, &status );
			printf( "get STATUS 2 = 0x%02X\n", status );
			break;

		case '3':

			cmd    = FLASH_READ_STATUS3;
			status = 0;
			ret = dirc->getStatus( cmd, &status );
			printf( "get STATUS 3 = 0x%02X\n", status );
			break;

		case '7':

			cmd = FLASH_WRITE_STATUS1;
			ret = dirc->setStatus( cmd, addr );
			printf( "set STATUS 1 = 0x%02X\n", addr );
			break;

		case '8':

			cmd = FLASH_WRITE_STATUS2;
			ret = dirc->setStatus( cmd, addr );
			printf( "set STATUS 2 = 0x%02X\n", addr );
			break;

		case '9':

			cmd = FLASH_WRITE_STATUS3;
			ret = dirc->setStatus( cmd, addr );
			printf( "set STATUS 3 = 0x%02X\n", addr );
			break;

		case 'q':
			printf("quit.\n");
			return 0;
			break;

		default:
			usage();
			break;
	}

	/* Close */
	dirc->spiClose();

  return 1;
}

FlashControl *dirc = new FlashControl();

int main( int argc, char **argv )
{
  (void)argc;
  (void)argv;

  usage();
  while( get_and_exec_cmd( dirc ) == 1 );

  return 0;
}
